/******************************************************************************
 *
 * AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: 
 * Simon Vey, Thomas Witkowski, Andreas Naumann, Simon Praetorius, et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * This file is part of AMDiS
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/
#include "Timer.h"
#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
#include <mpi.h>
#endif

using namespace boost::posix_time;

namespace AMDiS {
  
  Timer::Timer() : 
     first_seq(microsec_clock::local_time())
#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
    ,first_mpi(MPI::Wtime())
#endif
  {}

  void Timer::reset()
  {
#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
    first_mpi = MPI::Wtime();
#else
    first_seq = microsec_clock::local_time();
#endif
  }

  double Timer::elapsed()
  {
#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
    return MPI::Wtime() - first_mpi;
#else
    time_duration td = microsec_clock::local_time()-first_seq;
    return static_cast<double>(td.total_milliseconds())*1.e-3;
#endif
  }
  
} // end namespace AMDiS
