/******************************************************************************
 *
 * AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: 
 * Simon Vey, Thomas Witkowski, Andreas Naumann, Simon Praetorius, et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * This file is part of AMDiS
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/


/** \file VtkReader.h */

#ifndef AMDIS_VTKREADER_DETAIL_H
#define AMDIS_VTKREADER_DETAIL_H

// need some extension-methods/libs (pugixml, nanoflann)
#ifdef HAVE_EXTENSIONS

#include <cstring>
#include "DOFVector.h"
#include "Mesh.h"
#include "boost/filesystem.hpp"
#include "pugixml.hpp"
#include "kdtree_nanoflann.h"
#include "VectorOperations.h"

namespace AMDiS 
{
  namespace io 
  {
    namespace VtkReader 
    {
      namespace detail
      {

	inline void valueVector2type(std::vector<double> p, 
				    double& value)
	{
	  TEST_EXIT(p.size() != 0)("Not enough data for assignment!\n");
	  value = p[0];
	}

	
	inline void valueVector2type(std::vector<double> p, 
				    WorldVector<double>& value)
	{
	  TEST_EXIT(static_cast<int>(p.size()) == Global::getGeo(WORLD))("Not enough data for assignment!\n");
	  for (int i = 0; i < Global::getGeo(WORLD); i++)
	    value[i] = p[i];
	}

	
	inline void valueVector2type(std::vector<double> p, 
				    std::vector<double>& value)
	{
	  TEST_EXIT(p.size() != 0)("Not enough data for assignment!\n");
	  for (size_t i = 0; i < p.size(); i++)
	    value.push_back(p[i]);
	}

	
	inline void string2pointList(std::string& input, 
				    std::vector<WorldVector<double> >& pointList)
	{
	  int dow = Global::getGeo(WORLD);
	  int dowExplicit = 3;

	  std::vector<std::string> valueStringList;
	  std::stringstream ss(input);
	  std::string buf;
	  while (ss >> buf)
	    valueStringList.push_back(buf);

	  pointList.clear();

	  std::vector<std::string>::iterator it;
	  size_t j = 0;
	  for (it = valueStringList.begin(); it != valueStringList.end();j++) {
	    WorldVector<double> p;
	    for (int i = 0; i < dowExplicit && it != valueStringList.end(); i++, it++)
	      if (i < dow)
		p[i] = boost::lexical_cast<double>(*it);
	    pointList.push_back(p);
	  }
	}

	
	template<typename T>
	void string2valueList(std::string& input, 
				    std::vector<T>& valueList, 
				    int numComponent = 1, 
				    int numComponentMax = -1)
	{
	  if (numComponentMax < 0)
	    numComponentMax = numComponent;

	  std::vector<std::string> valueStringList;
	  std::stringstream ss(input);
	  std::string buf;
	  while (ss >> buf)
	    valueStringList.push_back(buf);

	  valueList.clear();

	  std::vector<std::string>::iterator it;
	  for (it = valueStringList.begin(); it != valueStringList.end();) {
	    std::vector<double> p;
	    for (int i = 0; i < numComponentMax && it != valueStringList.end(); i++, it++)
	      if (i < numComponent)
		p.push_back(boost::lexical_cast<double>(*it));
	    T value; valueVector2type(p, value);
	    valueList.push_back(value);
	  }
	}


	// find point in mesh using KD-tree structure
	inline size_t getNearestIndex(experimental::KD_Tree& tree, 
				      WorldVector<double>& x)
	{
	  // do a knn search
	  const size_t num_nnp = 1;
	  std::vector<size_t> ret_indexes(num_nnp);
	  std::vector<double> out_dists_sqr(num_nnp);

	  nanoflann::KNNResultSet<double> resultSet(num_nnp);
	  resultSet.init(&ret_indexes[0], &out_dists_sqr[0] );

	  tree.index->findNeighbors(resultSet, x.begin(), nanoflann::SearchParams(10));
	  return ret_indexes[0];
	}

      } // end namespace detail    
    } // end namespace VtkReader
  } // end namespace io
} // end namespace AMDiS

#endif // HAVE_EXTENSIONS

#endif // AMDIS_VTKREADER_DETAIL_H
