/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/


namespace meshconv2 {

  template<int dow>
  void coordinate_transform(double x[], double shift[], double scale[], double alpha, double beta)
  {
    mtl::dense2D<double> Rx(dow,dow),Ry(dow,dow),T(dow,dow),S(dow,dow);
    Rx = 1.0;
    Ry = 1.0;
    S = 1.0;
    if (dow == 3) {
      Rx(1,1) = cos(alpha);  Rx(1,2) = -sin(alpha);
      Rx(2,1) = sin(alpha);  Rx(2,2) = cos(alpha);

      Ry(0,0) = cos(beta);  Ry(0,2) = sin(beta);
      Ry(2,0) = -sin(beta); Ry(2,2) = cos(beta);

      S(0,0) = scale[0];  S(1,1) = scale[1];  S(2,2) = scale[2];
    } else if (dow == 2) {
      Rx(0,0) = cos(alpha);  Rx(0,1) = -sin(alpha);
      Rx(1,0) = sin(alpha);  Rx(1,1) = cos(alpha);

      Ry = 1.0;

      S(0,0) = scale[0];  S(1,1) = scale[1];
    } else {
      S(0,0) = scale[0];
    }

    T = Ry*Rx*S;

    mtl::dense_vector<double> coords(dow);
    for (int i = 0; i < dow; i++)
      coords[i] = x[i];

    coords = T*coords;

    for (int i = 0; i < dow; i++)
      x[i] = coords[i]+shift[i];
  }

} // end namespace meshconv2
