/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/


#ifndef EXTENSIONS_VALUE_TYPES_H
#define EXTENSIONS_VALUE_TYPES_H

#include <vector>
#include <list>
#include <set>

#include "DOFVector.h"
#include "AbstractFunction.h"

#include "boost/type_traits.hpp"

using namespace AMDiS;

template<typename T> struct DOFView; 

/// Type-Traits for value_type of Data-Structures
/// ________________________________________________________________________________________________

template<typename T, typename enable=void> struct ValueType { typedef T type; };
template<typename T> struct ValueType< DOFVector<T> > { typedef T type; };
template<typename T> struct ValueType< std::vector<T> > { typedef T type; };
template<typename T> struct ValueType< std::list<T> > { typedef T type; };
template<typename T> struct ValueType< std::set<T> > { typedef T type; };
template<typename T> struct ValueType< mtl::dense_vector<T> > { typedef T type; };
template<typename T> struct ValueType< WorldVector<T> > { typedef T type; };
template<typename Derived> struct ValueType< Derived, typename boost::enable_if<boost::is_base_of<DOFView<double>, Derived > >::type> { typedef double type; };
template<typename Derived> struct ValueType< Derived, typename boost::enable_if<boost::is_base_of<DOFView<WorldVector<double> >, Derived > >::type> { typedef WorldVector<double> type; };
template<typename Derived> struct ValueType< Derived, typename boost::enable_if<boost::is_base_of<AbstractFunction<double, WorldVector<double> >, Derived > >::type> { typedef double type; };
template<typename Derived> struct ValueType< Derived, typename boost::enable_if<boost::is_base_of<AbstractFunction<WorldVector<double>, WorldVector<double> >, Derived > >::type> { typedef WorldVector<double> type; };
  

/// random-accessible vectors
/// ________________________________________________________________________________________________

template<typename Vector> struct is_vector : public boost::false_type {};
template<typename T> struct is_vector< std::vector<T> > : public boost::true_type {};
template<typename T> struct is_vector< mtl::dense_vector<T> > : public boost::true_type {};
template<typename T> struct is_vector< WorldVector<T> > : public boost::true_type {};

/// random-accessible matrices
/// ________________________________________________________________________________________________

template<typename Matrix> struct is_matrix : public boost::false_type {};
template<typename T, typename Param> struct is_matrix< mtl::matrix::base_matrix<T, Param> > : public boost::true_type {};
template<typename T> struct is_matrix< mtl::matrix::dense2D<T> > : public boost::true_type {};
template<typename T> struct is_matrix< mtl::matrix::compressed2D<T> > : public boost::true_type {};
template<typename T> struct is_matrix< WorldMatrix<T> > : public boost::true_type {};



#endif // EXTENSIONS_VALUE_TYPES_H
