/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef BASE_PROBLEM_RB_H
#define BASE_PROBLEM_RB_H

#include "AMDiS.h"
#include "BaseProblem.h"
#include "time/ExtendedRosenbrockStationary.h"

using namespace AMDiS;

/** BaseProblems for Rosenbrock time-discretization
 */

class BaseProblem_RB : public BaseProblem<ExtendedRosenbrockStationary>
{
public: // typedefs

  typedef BaseProblem<ExtendedRosenbrockStationary> super;

// Rosenbrock methods
//_________________________________________________________________________

public:
  
  BaseProblem_RB(std::string name_, bool createProblem = true) : super(name_, createProblem), minus1(-1.0) {}
  
  Flag oneIteration(AdaptInfo *adaptInfo, Flag toDo) 
  {
    Flag flag = prob->oneIteration(adaptInfo, toDo);
    
    oldTimestep = adaptInfo->getTimestep();
    oldMeshChangeIdx = getMesh()->getChangeIndex();
    return flag;
  }
  
  Flag stageIteration(AdaptInfo *adaptInfo, Flag flag, bool asmMatrix, bool asmVector)
  {
    return prob->stageIteration(adaptInfo, flag, asmMatrix, asmVector);
  }
    
  double estimateTimeError(AdaptInfo* adaptInfo)
  {
    return prob->estimateTimeError(adaptInfo);
  }
  
  void acceptTimestep(AdaptInfo* adaptInfo) { prob->acceptTimestep(adaptInfo); }  
    
  double getNewTimestep(double tol, double tau, bool restrict = true)
  {
    return prob->getNewTimestep(tol, tau, restrict);
  }
    
  double getNewTimestep(double tol, double oldTau, double tau, bool restrict = true)
  {
    return prob->getNewTimestep(tol, oldTau, tau, restrict);
  }
  
  void reduceOrder(double tauRej, double tau) { prob->reduceOrder(tauRej, tau); }
  void setTau(double *ptr) { prob->setTau(ptr); }
  void setOldTime(double t) { prob->setOldTime(t); }
    
  /// get the i-th stage-solution-vector
  DOFVector<double> *getStageSolution(int i)
  { FUNCNAME("CoupledBaseProblem::getStageSolution(i,j)");
  
    TEST_EXIT(0<=i && i<getNumComponents())("Indices out of range!\n");
    return prob->getStageSolution(i);
  }  
  
  /// get the i-th unVec-vector
  DOFVector<double> *getUnVec(int i) 
  { FUNCNAME("CoupledBaseProblem::getUnVec(i)");
  
    TEST_EXIT(0<=i && i<getNumComponents())("Indices out of range!\n");
    return prob->getUnVec(i);
  }
  
protected:
  double minus1;
};

#endif // BASE_PROBLEM_RB_H
