/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef CAHN_HILLIARD_H
#define CAHN_HILLIARD_H

#include "AMDiS.h"
#include "BaseProblem.h"
#include "chns.h"
    
namespace detail {
  
  template<typename ProblemStatType>
  class CahnHilliard : public BaseProblem<ProblemStatType>
  {
  public: // definition of types

    typedef BaseProblem<ProblemStatType> super;
    typedef CahnHilliard<ProblemStatType> self;

  public: // public methods

    CahnHilliard(const std::string &name_);
    ~CahnHilliard() {};

    void solveInitialProblem(AMDiS::AdaptInfo *adaptInfo) override;

    double getEpsilon() { return eps; }
    int getDoubleWellType() { return doubleWell; }

    void fillOperators() override;
    void fillBoundaryConditions() override {}
    
    void finalizeData() override;

  protected: // protected variables

    bool useMobility;
    bool useReinit;

    unsigned dim;

    int doubleWell;

    double gamma;
    double eps;
    double minusEps;
    double epsInv;
    double minusEpsInv;
    double epsSqr;
    double minusEpsSqr;
  };

} // end namespace detail

#include "CahnHilliard.hh"

typedef ::detail::CahnHilliard<AMDiS::ProblemStat> CahnHilliard;


#endif // CAHN_HILLIARD_H
