/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef LINEAR_ELASTICITY_H
#define LINEAR_ELASTICITY_H

#include "AMDiS.h"
#include "BaseProblem.h"
#include "ExtendedProblemStat.h"

using namespace AMDiS;

/** \ingroup Elasticity
 * \brief
 * Elastodynamics – the wave equation
 */
class LinearElasticity : public BaseProblem<ExtendedProblemStat>
{
public: // typedefs
  
  typedef BaseProblem<ExtendedProblemStat> super;
  
public: // methods

  LinearElasticity(const std::string &name_);

  /// if initialVelocityIsSet is true delete initialX and initialY
  ~LinearElasticity();

  virtual void initData();
  virtual void solveInitialProblem(AdaptInfo *adaptInfo);

  virtual void transferInitialSolution(AdaptInfo *adaptInfo);  

//   virtual void initTimestep(AdaptInfo *adaptInfo);
  /// Implementation of ProblemTimeInterface::closeTimestep().
  virtual void closeTimestep(AdaptInfo *adaptInfo);

  // === getting/setting methods ===

  virtual void fillOperators();
  virtual void fillBoundaryConditions();

protected: // variables

  double mu;
  double lambda;
  double rho;
  
  int nonLinTerm;

  WorldVector<double> force;
  DOFVector<WorldVector<double> >* displacement;

  FileVectorWriter *fileWriter;
};
#endif // LINEAR_ELASTICITY_H
