/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef NAVIER_STOKES_PHASE_CHORIN_H
#define NAVIER_STOKES_PHASE_CHORIN_H

#include "MyGlobal.h"
#include "NavierStokes_Chorin.h"
#include "Helpers.h"
#include "POperators.h"
#include "BoundaryFunctions.h"
#include "Refinement.h"
#include "MeshFunction_Level.h"
#include "MyFileWriter.h"
#include "DiffuseDomain.h"
#include <boost/numeric/mtl/mtl.hpp>
#include <boost/numeric/mtl/utility/property_map.hpp>


#define LINKS 1
#define RECHTS 2
#define UNTEN 3
#define OBEN 4

using namespace AMDiS;

/** \ingroup NavierStokesPhase_Chorin
 * \brief
 * Navier-Stokes problem in diffuse domain, using projection method {
 *       a.) Burgers equation (predictortProb)
 *       b.) Pressure Poisson problem
 *       c.) Burgers equation including pressure (predictorProb)
 *       d.) Pressure Poisson problem
 *       e.) Velocity update (correctorProb)
 * }
 */
class NavierStokesPhase_Chorin : public NavierStokes_Chorin
{
public: // typedefs

  typedef NavierStokes_Chorin super;

public: // methods

  NavierStokesPhase_Chorin(const std::string &name_, WorldVector<AbstractFunction<double, WorldVector<double> >* > initialVelocityFcts_, DiffuseDomain* geometryProb);

  virtual void solveInitialProblem(AdaptInfo *adaptInfo);

  /// Implementation of ProblemTimeInterface::initTimestep().
  virtual void initTimestep(AdaptInfo *adaptInfo);

  /// Implementation of ProblemTimeInterface::closeTimestep().
  virtual void closeTimestep(AdaptInfo *adaptInfo);

protected: // methods

  virtual void fillOperators();
  virtual void fillBoundaryConditions();

  virtual void addLaplaceTerm(int i);

protected: // variables

  DiffuseDomain *geometryProb;
  MyFileWriter *writer;

  double beta;
  double epsPhase2;

private:

};
#endif // NAVIER_STOKES_CHORIN_H
