/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef NAVIER_STOKES_TAYLOR_HOOD_RB_H
#define NAVIER_STOKES_TAYLOR_HOOD_RB_H

#include "AMDiS.h"
#include "Helpers.h"
#include "POperators.h"
#include "BaseProblem_RB.h"

using namespace AMDiS;

/** \ingroup NavierStokes_TaylorHood
 * \brief
 * Navier-Stokes problem, using Taylor Hood elements and
 * Rosenbrock method for time-discretization
 */
class NavierStokes_TaylorHood_RB : public BaseProblem_RB
{
public: // typedefs
  
  typedef BaseProblem_RB super;
  
public: // methods

  NavierStokes_TaylorHood_RB(const std::string &name_, bool createProblem = true);

  ~NavierStokes_TaylorHood_RB();

  void initData() override;

  void transferInitialSolution(AdaptInfo *adaptInfo) override;  

  void closeTimestep(AdaptInfo *adaptInfo) override;

  // === getting/setting methods ===

  DOFVector<WorldVector<double> >* getVelocity()
  {
    TEST_EXIT_DBG(velocity != NULL)
      ("velocity if NULL!\n");
    return velocity;
  }

  void fillOperators() override;

  virtual void addLaplaceTerm(int i);

protected: // variables

  DOFVector<WorldVector<double> >* velocity;

  void calcVelocity()
  {
    if (dow == 1)
      transformDOF(prob->getSolution()->getDOFVector(0), 
		   velocity, new AMDiS::Vec1WorldVec<double>);
    else if (dow == 2)
      transformDOF(prob->getSolution()->getDOFVector(0), 
		   prob->getSolution()->getDOFVector(1), 
                   velocity, new AMDiS::Vec2WorldVec<double>);
    else if (dow == 3)
      transformDOF(prob->getSolution()->getDOFVector(0), 
		   prob->getSolution()->getDOFVector(1), 
		   prob->getSolution()->getDOFVector(2), 
		   velocity, new AMDiS::Vec3WorldVec<double>);
  }
  
  int laplaceType;

  double oldTimestep;
  double viscosity;

  WorldVector<double> force;

  FileVectorWriter *fileWriter;
};
#endif // NAVIER_STOKES_TAYLOR_HOOD_RB_H

