/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef PHASE_FIELD_CRYSTAL_H
#define PHASE_FIELD_CRYSTAL_H

#include "AMDiS.h"
#include "BaseProblem.h"
#include "ExtendedProblemStat.h"

using namespace AMDiS;

/** Phase-field Crystal problem
*/
class PhaseFieldCrystal : public BaseProblem<ExtendedProblemStat>
{
public: // typedefs

  typedef BaseProblem<ExtendedProblemStat> super;

public: // methods

  PhaseFieldCrystal(const std::string &name_, bool createProblem = true);
  ~PhaseFieldCrystal() {};

  void fillOperators() override;
  void finalizeData() override;
  
  void setDensity(double density_)
  {
    density = density_;
  }

protected: // methods
  
  double* getTempParameter() { return &tempParameter; }

protected: // variables

  bool useMobility;

  double tempParameter;
  double r;
  double q;
  double q2;
  double rho0;
  double density;
  double two;
  double twoQ2;
  double minus2;
};


/** \ingroup MainInstat
 * \brief
 * Abstract function to calculate the pure PFC-Energy
 */
class Energy : public BinaryAbstractFunction<double,double,double>
{
  public:
    Energy() : BinaryAbstractFunction<double,double,double>(4) { }

    double operator()(const double &rho, const double &mu) const {
      return -0.25*sqr(sqr(rho)) + 0.5*rho*mu; }
};


class MobilityPfc : public AbstractFunction<double,double>
{
  public:
    MobilityPfc(double density_ = -0.3, double factor_ = 1.0, int degree=1) : 
      AbstractFunction<double,double>(degree+1),
      density(density_),
      factor(factor_),
      delta(1.e-6) { }

    double operator()(const double &rho) const 
    {
      double mobility= std::abs(rho + 1.5)*factor;
      return std::max(mobility, 0.0);
    }
  
  protected:
    double density;
    double factor;
    double delta;
};

#endif // PHASE_FIELD_CRYSTAL_H
