#include "QuasiCrystal.h"

using namespace std;
using namespace AMDiS;

QuasiCrystal::QuasiCrystal(const std::string &name_) :
  super(name_),
  q(2.0*cos(m_pi/12.0)),
  r(-0.015),
  c(50.0),
  alpha(1.0),
  density(0.0),	// mean density
  two(2.0),
  minus2(-2.0),
  three(3.0),
  minus3(-3.0)
{  
  Parameters::get(name + "->r",r);
  Parameters::get(name + "->c", c);
  Parameters::get(name + "->q", q);
  Parameters::get(name + "->density", density);
  Parameters::get(name + "->alpha", alpha);
};


void QuasiCrystal::fillOperators()
{ FUNCNAME("QuasiCrystal::fillOperators()");
  
  // 
  Operator *opM = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opM->addTerm(new Simple_ZOT);
  Operator *opMold = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMold->addTerm(new VecAtQP_ZOT(prob->getSolution()->getDOFVector(0), NULL));

  // 
  Operator *opL = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opL->addTerm(new Simple_SOT);
  Operator *opL1 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opL1->addTerm(new Simple_SOT(2.0*sqr(q)*c*(1.0+sqr(q))));
  Operator *opL2 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opL2->addTerm(new Simple_SOT(sqr(1.0+sqr(q))+2.0*sqr(q)));
  Operator *opL3 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opL3->addTerm(new Simple_SOT(2.0*(1.0+sqr(q))));
    
  // 
  Operator *opLin = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opLin->addZeroOrderTerm(new Simple_ZOT(-c*sqr(sqr(q))-r));
  
  Operator *opPow1 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opPow1->addZeroOrderTerm(new VecAtQP_ZOT(prob->getSolution()->getDOFVector(0)));
  // 
  Operator *opPow2 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opPow2->addZeroOrderTerm(new VecAtQP_ZOT(prob->getSolution()->getDOFVector(0),new Pow2Functor(1.0)));
  // 
  Operator *opPow3 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opPow3->addZeroOrderTerm(new VecAtQP_ZOT(prob->getSolution()->getDOFVector(0),new Pow3Functor(1.0)));
  
  // mu - c*laplace(nu) - laplace1(rho) + lin(rho) + [2rho'-3(rho')^2]*rho = (rho')^2-2*(rho')^3
  // --------------------------------------------------------------------------------------------
  prob->addMatrixOperator(opL1, 0, 0); // -laplace1(rho)
  prob->addMatrixOperator(opLin, 0, 0); // lin(rho)
  prob->addMatrixOperator(opPow1, 0, 0, &two, &two); // 2rho' * rho
  prob->addMatrixOperator(opPow2, 0, 0, &minus3, &minus3); // -3(rho')^2 * rho
  
  prob->addMatrixOperator(opM, 0, 1); // mu
  prob->addMatrixOperator(opL, 0, 2, &c, &c); // -c*laplace(nu)
  
  prob->addVectorOperator(opPow2, 0);
  prob->addVectorOperator(opPow3, 0, &minus2, &minus2);
  
  setAssembleMatrixOnlyOnce_butTimestepChange(0,1);
  setAssembleMatrixOnlyOnce_butTimestepChange(0,2);
  
  // dt(rho) - laplace(mu) = 0
  // -----------------------------------
  prob->addMatrixOperator(opM, 1, 0, getInvTau(), getInvTau());
  prob->addMatrixOperator(opL, 1, 1); // -laplace(mu)
  // . . . vectorOperators . . . . . . . . . . . . . . .
  prob->addVectorOperator(opMold, 1, getInvTau(), getInvTau());
  
  setAssembleMatrixOnlyOnce_butTimestepChange(1,0);
  setAssembleMatrixOnlyOnce_butTimestepChange(1,1);
  
  // nu - laplace2(rho) - laplace(psi) = 0
  // ----------------------------------------------------------------------
  prob->addMatrixOperator(opL2, 2, 0); // -laplace2(rho)
  prob->addMatrixOperator(opM, 2, 2); // nu
  prob->addMatrixOperator(opL, 2, 3); // -laplace(psi)
  
  setAssembleMatrixOnlyOnce_butTimestepChange(2,0);
  setAssembleMatrixOnlyOnce_butTimestepChange(2,2);
  setAssembleMatrixOnlyOnce_butTimestepChange(2,3);
  
  // psi - laplace3(rho) - laplace(theta) = 0
  // ----------------------------------------------------------------------
  prob->addMatrixOperator(opL3, 3, 0); // -laplace3(rho)
  prob->addMatrixOperator(opM, 3, 3); // psi
  prob->addMatrixOperator(opL, 3, 4); // -laplace(theta)
  
  setAssembleMatrixOnlyOnce_butTimestepChange(3,0);
  setAssembleMatrixOnlyOnce_butTimestepChange(3,3);
  setAssembleMatrixOnlyOnce_butTimestepChange(3,4);
  
  // theta - laplace(rho) = 0
  // ----------------------------------------------------------------------
  prob->addMatrixOperator(opL, 4, 0); // -laplace(rho)
  prob->addMatrixOperator(opM, 4, 4); // theta
  
  setAssembleMatrixOnlyOnce_butTimestepChange(4,0);
  setAssembleMatrixOnlyOnce_butTimestepChange(4,4);
};


void QuasiCrystal::fillBoundaryConditions()
{ FUNCNAME("QuasiCrystal::fillBoundaryConditions()");
};
