/** \file QuasiCrystal.h */

#ifndef QUASI_CRYSTAL_H
#define QUASI_CRYSTAL_H

#include "AMDiS.h"
#include "BaseProblem.h"
#include "ExtendedProblemStat.h"

using namespace AMDiS;

/** Phase-field Crystal problem
 */
class QuasiCrystal : public BaseProblem<ExtendedProblemStat>
{
public: // typedefs

  typedef BaseProblem<ExtendedProblemStat> super;

public:

  QuasiCrystal(const std::string &name_);
  ~QuasiCrystal() {};

  virtual void fillOperators();
  virtual void fillBoundaryConditions();

protected:

  bool useMobility;

  double q;
  double r;
  double c;
  double alpha;
  double density;
  
  double two;
  double minus2;
  double three;
  double minus3;
};


/** \ingroup MainInstat
 * \brief
 * Abstract function to calculate the pure PFC-Energy
 */
// class Energy : public BinaryAbstractFunction<double,double,double>
// {
//   public:
//     Energy() : BinaryAbstractFunction<double,double,double>(4) { }
// 
//     double operator()(const double &rho, const double &mu) const {
//       return -0.25*sqr(sqr(rho)) + 0.5*rho*mu; }
// };


class Pow2Functor : public AbstractFunction<double,double>
{
  public:
    Pow2Functor(double factor_=1.0) :
      AbstractFunction<double,double>(2),
      factor(factor_) { }
    double operator()(const double &ch) const
    {
      return factor * sqr(ch);
    }

  protected:
    double factor;
};


class Pow3Functor : public AbstractFunction<double,double>
{
  public:
    Pow3Functor(double factor_=1.0) :
      AbstractFunction<double,double>(3),
      factor(factor_) { }
    double operator()(const double &ch) const
    {
      return factor * sqr(ch) * ch;
    }

  protected:
    double factor;
};

#endif // PHASE_FIELD_CRYSTAL_H
