#include "QuasiCrystal_RB.h"

using namespace std;
using namespace AMDiS;

QuasiCrystal_RB::QuasiCrystal_RB(const std::string &name_) :
  super(name_),
  q(2.0*cos(m_pi/12.0)),
  r(-0.015),
  c(50.0),
  minusC(-50.0),
  alpha(1.0),
  density(0.0),	// mean density
  two(2.0),
  minus2(-2.0),
  three(3.0),
  minus3(-3.0)
{  
  Parameters::get(name + "->r",r);
  Parameters::get(name + "->c", c);
  Parameters::get(name + "->q", q);
  Parameters::get(name + "->density", density);
  Parameters::get(name + "->alpha", alpha);
  minusC = -c;
};


void QuasiCrystal_RB::fillOperators()
{ FUNCNAME("QuasiCrystal_RB::fillOperators()");
  
  // mass matrices
  Operator *opM = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opM->addTerm(new Simple_ZOT);
  Operator *opM_1 = new Operator(prob->getFeSpace(1), prob->getFeSpace(1));
  opM_1->addTerm(new VecAtQP_ZOT(prob->getStageSolution(1), NULL));
  Operator *opM_2 = new Operator(prob->getFeSpace(2), prob->getFeSpace(2));
  opM_2->addTerm(new VecAtQP_ZOT(prob->getStageSolution(2), NULL));
  Operator *opM_3 = new Operator(prob->getFeSpace(3), prob->getFeSpace(3));
  opM_3->addTerm(new VecAtQP_ZOT(prob->getStageSolution(3), NULL));
  Operator *opM_4 = new Operator(prob->getFeSpace(4), prob->getFeSpace(4));
  opM_4->addTerm(new VecAtQP_ZOT(prob->getStageSolution(4), NULL));

  // stiffness matrices
  Operator *opL_0 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opL_0->addTerm(new Simple_SOT);
  opL_0->setUhOld(prob->getStageSolution(0));
  Operator *opL_1 = new Operator(prob->getFeSpace(1), prob->getFeSpace(1));
  opL_1->addTerm(new Simple_SOT);
  opL_1->setUhOld(prob->getStageSolution(1));
  Operator *opL_2 = new Operator(prob->getFeSpace(2), prob->getFeSpace(2));
  opL_2->addTerm(new Simple_SOT);
  opL_2->setUhOld(prob->getStageSolution(2));
  Operator *opL_3 = new Operator(prob->getFeSpace(3), prob->getFeSpace(3));
  opL_3->addTerm(new Simple_SOT);
  opL_3->setUhOld(prob->getStageSolution(3));
  Operator *opL_4 = new Operator(prob->getFeSpace(4), prob->getFeSpace(4));
  opL_4->addTerm(new Simple_SOT);
  opL_4->setUhOld(prob->getStageSolution(4));
  
  Operator *opL1 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opL1->addTerm(new Simple_SOT(2.0*sqr(q)*c*(1.0+sqr(q))));
  Operator *opL2 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opL2->addTerm(new Simple_SOT(sqr(1.0+sqr(q))+2.0*sqr(q)));
  Operator *opL3 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opL3->addTerm(new Simple_SOT(2.0*(1.0+sqr(q))));
    
  // 
  Operator *opLin = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opLin->addZeroOrderTerm(new Simple_ZOT(-c*sqr(sqr(q))-r));
  Operator *opLin_0 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opLin_0->addZeroOrderTerm(new VecAtQP_ZOT(prob->getStageSolution(0), new AMDiS::Factor<double>(-c*sqr(sqr(q))-r)));
  
  Operator *opPow1 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opPow1->addZeroOrderTerm(new VecAtQP_ZOT(prob->getStageSolution(0)));
  // 
  Operator *opPow2 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opPow2->addZeroOrderTerm(new VecAtQP_ZOT(prob->getStageSolution(0),new AMDiS::Pow<2>(1.0)));
  // 
  Operator *opPow3 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opPow3->addZeroOrderTerm(new VecAtQP_ZOT(prob->getStageSolution(0),new AMDiS::Pow<3>(1.0)));
  
  
  Operator *opPow1_ = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opPow1->addZeroOrderTerm(new VecAtQP_ZOT(prob->getUnVec(0)));
  // 
  Operator *opPow2_ = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opPow2->addZeroOrderTerm(new VecAtQP_ZOT(prob->getUnVec(0),new AMDiS::Pow<2>(1.0)));
  
  // 0 = mu - c*laplace(nu) - laplace1(rho) + lin(rho) + rho^2 - rho^3
  // --------------------------------------------------------------------------------------------
  
  opL1->setUhOld(prob->getStageSolution(0));
  prob->addVectorOperator(opL1, 0);
  prob->addMatrixOperator(opL1, 0, 0, &minus1, &minus1); // -laplace1(rho)
  
  prob->addVectorOperator(opLin_0, 0);
  prob->addMatrixOperator(opLin, 0, 0, &minus1, &minus1); // lin(rho)
  
  prob->addVectorOperator(opM_1, 0);
  prob->addMatrixOperator(opM, 0, 1, &minus1, &minus1); // mu
  
  prob->addVectorOperator(opL_2, 0, &c, &c);
  prob->addMatrixOperator(opL_2, 0, 2, &minusC, &minusC); // -c*laplace(nu)
  
  // nonlinear term
  
  prob->addVectorOperator(opPow2, 0); 				// rho^2
  prob->addVectorOperator(opPow3, 0, &minus1, &minus1); 		// -rho^3
  
  prob->addMatrixOperator(opPow1_, 0, 0, &minus2, &minus2);
  prob->addMatrixOperator(opPow2_, 0, 0, &three, &three);
  
  // dt(rho) = laplace(mu)
  // -----------------------------------
  prob->addTimeOperator(1, 0);		// dt(rho)
  
  prob->addVectorOperator(opL_1, 1, &minus1, &minus1);
  prob->addMatrixOperator(opL_1, 1, 1); 	// -laplace(mu)
  
  // 0 = nu - laplace2(rho) - laplace(psi)
  // ----------------------------------------------------------------------
  opL2->setUhOld(prob->getStageSolution(0));
  prob->addVectorOperator(opL2, 2);
  prob->addMatrixOperator(opL2, 2, 0, &minus1, &minus1); // -laplace2(rho)
  
  prob->addVectorOperator(opM_2, 2);
  prob->addMatrixOperator(opM, 2, 2, &minus1, &minus1); // nu
  
  prob->addVectorOperator(opL_3, 2);
  prob->addMatrixOperator(opL_3, 2, 3, &minus1, &minus1); // -laplace(psi)
  
  // 0 = psi - laplace3(rho) - laplace(theta)
  // ----------------------------------------------------------------------
  
  opL3->setUhOld(prob->getStageSolution(0));
  prob->addVectorOperator(opL3, 3);
  prob->addMatrixOperator(opL3, 3, 0, &minus1, &minus1); // -laplace3(rho)
  
  prob->addVectorOperator(opM_3, 3);
  prob->addMatrixOperator(opM, 3, 3, &minus1, &minus1); // psi
  
  prob->addVectorOperator(opL_4, 3);
  prob->addMatrixOperator(opL_4, 3, 4, &minus1, &minus1); // -laplace(theta)
  
  // 0 = theta - laplace(rho)
  // ----------------------------------------------------------------------
  prob->addVectorOperator(opL_0, 4);
  prob->addMatrixOperator(opL_0, 4, 0, &minus1, &minus1); // -laplace(rho)
  
  prob->addVectorOperator(opM_4, 4);
  prob->addMatrixOperator(opM, 4, 4, &minus1, &minus1); // theta
};


void QuasiCrystal_RB::fillBoundaryConditions()
{ FUNCNAME("QuasiCrystal_RB::fillBoundaryConditions()");
};
