/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef AMDIS_PETSC_PRECON_CH_H
#define AMDIS_PETSC_PRECON_CH_H

#include "solver/PetscSolver.h"

namespace AMDiS {

  using namespace std;

  struct ChData {
    ChData() : matM(PETSC_NULL), matMinusDeltaK(PETSC_NULL) {}
    KSP kspM, kspMpK;
    Mat matM, matMinusDeltaK;
    double delta, eps;
  };

  class PetscPreconCahnHilliard : public PetscRunner
  {
  public:
    typedef PetscPreconCahnHilliard self;
    
    PetscPreconCahnHilliard(LinearSolver* oem_)
    : PetscRunner(oem_),
      MpK(PETSC_NULL),
      eps(NULL),
      delta(NULL) { }

    void setData(double *epsPtr, double *deltaPtr)
    {
      eps = epsPtr;
      delta = deltaPtr;
    }

    void init(const BlockMatrix& A, const Mat& fullMatrix);
    void exit();

  private:
    Mat MpK;
    ChData data;

    double *eps;
    double *delta;
  };

}

#endif // AMDIS_PETSC_PRECON_PFC_H
