/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef AMDIS_PETSC_PRECON_PFC_H
#define AMDIS_PETSC_PRECON_PFC_H

#include "parallel/PetscSolverGlobalBlockMatrix.h"

namespace AMDiS { namespace Parallel {

  struct PfcData {
    KSP kspM, kspM2, kspMpK, kspSchur;
    Mat matM, matK;
    double delta, tau;
  };

  class PetscSolverPfc : public PetscSolverGlobalBlockMatrix
  {
  public:
    /// Creator class
    class Creator : public LinearSolverCreator
    {
    public:
      virtual ~Creator() {}

      /// Returns a new PetscSolver object.
      LinearSolver* create() 
      { 
	return new PetscSolverPfc(this->name); 
      }
    };
    
    PetscSolverPfc(std::string name)
    : PetscSolverGlobalBlockMatrix(name),
      solverM(NULL),
      solverM2(NULL),
      solverK(NULL),
      solverMpK(NULL),
      useOldInitialGuess(false),
      tau(NULL)
    { 	
      Parameters::get(initFileStr + "->use old initial guess", useOldInitialGuess);
    }

    void setData(double *tauPtr)
    {
      tau = tauPtr;
    }

  protected:
    void initSolver(KSP &ksp);

    void initPreconditioner(PC pc);
    void exitPreconditioner(PC pc);
    
    PetscSolver* createSubSolver(int component, std::string kspPrefix);

  private:
    Mat matMpK;
    Mat matSchur;
    PfcData data;

    PetscSolver *solverM, *solverM2, *solverK, *solverMpK;

    bool useOldInitialGuess;    
    SystemVector* solution;

    double *tau;
  };

} }

#endif // AMDIS_PETSC_PRECON_PFC_H
