/** \file PhaseFieldCrystal_.h */

#ifndef PHASE_FIELD_CRYSTAL_PRECON_H_
#define PHASE_FIELD_CRYSTAL_PRECON_H_

#include "AMDiS.h"
#include "BaseProblem.h"
#include "ExtendedProblemStat.h"

using namespace AMDiS;

/** Phase-field Crystal problem
 */
class PhaseFieldCrystal_ : public BaseProblem<ExtendedProblemStat>
{
public: // typedefs

  typedef BaseProblem<ExtendedProblemStat> super;

public:
 
  PhaseFieldCrystal_(const std::string &name_, bool createProblem = true);
  ~PhaseFieldCrystal_() {}

  double *getTempParameter() { return &tempParameter; }

  virtual void fillOperators();
  virtual void fillBoundaryConditions();

protected:

  bool useMobility;

  double tempParameter;
  double r;
  double rho0;
  double density;
  double two;
  double minus2;
};


/** \ingroup MainInstat
 * \brief
 * Abstract function to calculate the pure PFC-Energy
 */
class Energy : public BinaryAbstractFunction<double,double,double>
{
  public:
    Energy() : BinaryAbstractFunction<double,double,double>(4) { }

    double operator()(const double &rho, const double &mu) const {
      return -0.25*sqr(sqr(rho)) + 0.5*rho*mu; }
};


class MobilityPfc : public AbstractFunction<double,double>
{
  public:
    MobilityPfc(double density_ = -0.3, double factor_ = 1.0, int degree=1) : 
      AbstractFunction<double,double>(degree+1),
      density(density_),
      factor(factor_),
      delta(1.e-6) { }

    double operator()(const double &rho) const 
    {
      double mobility= std::abs(rho + 1.5)*factor;
      return std::max(mobility, 0.0);
    }
  
  protected:
    double density;
    double factor;
    double delta;
};

#endif // PHASE_FIELD_CRYSTAL_PRECON_H_
